/*
  S12232ZA_LEVEL_METER1.ino

  This sketch is for a level meter curcuit controlled by Arduino Uno.
  
  This sketch was developed by Synapse(Hiroshi Tanigawa) in 2013. This sketch is originally
  distributed at Team Schaft's Homepage. <http://www3.big.or.jp/~schaft/hardware>
  
  This sketch is free software: you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation, either version 2.1 of the License, or
  (at your option) any later version.
  
  This sketch is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this sketch.  If not, see <http://www.gnu.org/licenses/>.

*/

#include <TimerOne.h>
#include <MGLCD.h>
#include "kyoto.h"

#define CHARGE_PUMP_OSC_PIN 3

#define DIVISOR 8
#define WAVE_WIDTH 53

const TLcdPinAssignTable PinAssignTable={
    A0_DI  : A3, // A0
    CS1_E1 : A0, // E1
    CS2_E2 : A1, // E2
    E      : MGLCD_UNUSED_PIN,
    RW     : A2,
    DB0    : 8 ,
    DB1    : 9 ,
    DB2    : 10,
    DB3    : 11,
    DB4    : 4 ,
    DB5    : 5 ,
    DB6    : 6,
    DB7    : 7
}; // PinAssignTable;

MGLCD_S12232ZA MGLCD(PinAssignTable);
volatile unsigned char TimerCnt=0;
unsigned char LastCnt;
unsigned char CurrentCnt;

void timerIsr()
{
  TimerCnt++;
} // timerIsr

void PrintSlowly(const char *p)
{
  while(*p!='\0') {
    MGLCD.print(*(p++));
    delay(100);
  } // while
} // PrintSlowly

void opening()
{
  MGLCD.ClearScreen();
  for(int i=90; i>=0; i--) {
    MGLCD.DrawBitmap(BMP_kyoto,i,1,MGLCD_ROM);
    MGLCD.Line(i,0,i,31,0);
    delay(1);
  } // int i
  
  MGLCD.SetInvertMode(MGLCD_INVERT);
  MGLCD.Locate(7,0); PrintSlowly(" PHONE METER ");
  MGLCD.SetInvertMode(MGLCD_NON_INVERT);
  MGLCD.Locate(7,1); PrintSlowly("This software");
  MGLCD.Locate(7,2); PrintSlowly("was designed");
  MGLCD.Locate(7,3); PrintSlowly("in Kyoto.");
  
  delay(3000);
  MGLCD.ClearScreen();  
} // opening

void setup()
{
  // Enable Oscillator for charge pump
  pinMode(CHARGE_PUMP_OSC_PIN,OUTPUT);
  analogWrite(CHARGE_PUMP_OSC_PIN,127);
  
  // Enable pull-up registers of unused ports
  pinMode( 2,INPUT_PULLUP);
  pinMode(12,INPUT_PULLUP);
  
  // Turn on LED on port 13
  pinMode(13,OUTPUT);
  digitalWrite(13,HIGH);
  
  // Initialize Timer1
  Timer1.initialize(50000); // Set length 50ms
  Timer1.attachInterrupt(timerIsr);

  // Initialize LCD
  MGLCD.Reset();
  opening();
  MGLCD.Locate( 0,1); MGLCD.print('L');
  MGLCD.Locate(10,1); MGLCD.print('R');
  
  LastCnt=TimerCnt;
} // setup

void loop()
{
  int wave[2][WAVE_WIDTH];
  int WaveMin[2];
  int WaveMax[2];
  
  // Wait
  do {
    CurrentCnt=TimerCnt;;
  } while(CurrentCnt==LastCnt);
  LastCnt=CurrentCnt;
  
  // Get waves
  for(int i=0; i<WAVE_WIDTH; i++) {    
    wave[0][i]=(analogRead(A4)-512)/DIVISOR;
    wave[1][i]=(analogRead(A5)-512)/DIVISOR;
  } // for i

  // Clip wave and calculate WaveMin / WaveMax
  WaveMin[0]=WaveMin[1]= 10000;
  WaveMax[0]=WaveMax[1]=-10000;  
  for(int i=0; i<WAVE_WIDTH; i++) {    
    for(int j=0; j<2; j++) {
      if(wave[j][i]> 14) wave[j][i]= 14;
      if(wave[j][i]<-14) wave[j][i]=-14;
      if(wave[j][i]<WaveMin[j]) WaveMin[j]=wave[j][i];
      if(wave[j][i]>WaveMax[j]) WaveMax[j]=wave[j][i];
    } // for j
  } // for j
  
  // Display wave
  for(int i=0; i<WAVE_WIDTH; i++) {
    MGLCD.Line( 6+i,0, 6+i,28,0);
    MGLCD.Line(66+i,0,66+i,28,0);
    
    if(i>0) {
      MGLCD.Line( 5+i,14-wave[0][i-1], 6+i,14-wave[0][i]);
      MGLCD.Line(65+i,14-wave[1][i-1],66+i,14-wave[1][i]);      
    } // if
  } // for  
  
  // Display bar graphs
  for(int i=0; i<2; i++) {
    int val;
    
    val=(WaveMax[i]-WaveMin[i])*2;
    if(val>54) val=54;
    MGLCD.FillRect(6+i*60,29,6+i*60+val,31);
    MGLCD.FillRect(7+i*60+val,29,59+i*60,31,0);
  } // for i  
} // loop

