/*
  S12232ZA_LEVEL_METER2.ino

  This sketch is for a level meter curcuit controlled by Arduino Uno.
  
  This sketch was developed by Synapse(Hiroshi Tanigawa) in 2013. This sketch is originally
  distributed at Team Schaft's Homepage. <http://www3.big.or.jp/~schaft/hardware>
  
  This sketch is free software: you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation, either version 2.1 of the License, or
  (at your option) any later version.
  
  This sketch is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this sketch.  If not, see <http://www.gnu.org/licenses/>.

*/

#include <MGLCD.h>
#include "L.h"
#include "R.h"
#include "kyoto.h"

#define CHARGE_PUMP_OSC_PIN 3
#define WAVE_WIDTH 100

const TLcdPinAssignTable PinAssignTable={
    A0_DI  : A3, // A0
    CS1_E1 : A0, // E1
    CS2_E2 : A1, // E2
    E      : MGLCD_UNUSED_PIN,
    RW     : A2,
    DB0    : 8 ,
    DB1    : 9 ,
    DB2    : 10,
    DB3    : 11,
    DB4    : 4 ,
    DB5    : 5 ,
    DB6    : 6,
    DB7    : 7
}; // PinAssignTable;

MGLCD_S12232ZA MGLCD(PinAssignTable);
int BmpWidth;
int MaxBarWidth;

void PrintSlowly(const char *p)
{
  while(*p!='\0') {
    MGLCD.print(*(p++));
    delay(100);
  } // while
} // PrintSlowly

void opening()
{
  MGLCD.ClearScreen();
  for(int i=90; i>=0; i--) {
    MGLCD.DrawBitmap(BMP_kyoto,i,1,MGLCD_ROM);
    MGLCD.Line(i,0,i,31,0);
    delay(1);
  } // int i
  
  MGLCD.SetInvertMode(MGLCD_INVERT);
  MGLCD.Locate(7,0); PrintSlowly(" PHONE METER ");
  MGLCD.SetInvertMode(MGLCD_NON_INVERT);
  MGLCD.Locate(7,1); PrintSlowly("This software");
  MGLCD.Locate(7,2); PrintSlowly("was designed");
  MGLCD.Locate(7,3); PrintSlowly("in Kyoto.");
  
  delay(3000);
  MGLCD.ClearScreen(); 
} // opening

int dB(unsigned x)
{
  int result;
  unsigned level;
  
  result=0;
  level=1;
  while(true) {
    if(x<(level*18U)>>4) {
      return result;
    } else if(x<(level*20U)>>4) {
      return result+1;
    } else if(x<(level*23U)>>4) {
      return result+2;
    } else if(x<(level*25U)>>4) {
      return result+3;
    } else if(x<(level*29U)>>4) {
      return result+4;
    } else if(x<level*2) {
      return result+5;
    } else {
      result+=6;
      level<<=1;
    }
  } // while
} // dB

void setup()
{
  // Enable Oscillator for charge pump
  pinMode(CHARGE_PUMP_OSC_PIN,OUTPUT);
  analogWrite(CHARGE_PUMP_OSC_PIN,127);
  
  // Enable pull-up registers of unused ports
  pinMode( 2,INPUT_PULLUP);
  pinMode(12,INPUT_PULLUP);
  
  // Turn on LED on port 13
  pinMode(13,OUTPUT);
  digitalWrite(13,HIGH);
  
  // Initialize variables
  BmpWidth=pgm_read_byte(BMP_L);
  MaxBarWidth=122-(BmpWidth+2);
  
  // Initialize LCD
  MGLCD.Reset();
  opening();
  MGLCD.DrawBitmap(BMP_L,0, 2,MGLCD_ROM);
  MGLCD.DrawBitmap(BMP_R,0,18,MGLCD_ROM);
} // setup

void loop()
{
  unsigned wave[2][WAVE_WIDTH];
  unsigned WaveMin[2];
  unsigned WaveMax[2];
  int BarWidth;
  static int PeakVal[2]={0,0};
  static int PeakCnt[2]={0,0};

  // Get waves
  for(int i=0; i<WAVE_WIDTH; i++) {    
    wave[0][i]=analogRead(A4);
    wave[1][i]=analogRead(A5);
  } // for i

  // Calculate WaveMin / WaveMax
  WaveMin[0]=WaveMin[1]=10000;
  WaveMax[0]=WaveMax[1]=0; 
  for(int i=0; i<2; i++) { 
    for(int j=0; j<WAVE_WIDTH; j++) {    
      if(wave[i][j]<WaveMin[i]) WaveMin[i]=wave[i][j];
      if(wave[i][j]>WaveMax[i]) WaveMax[i]=wave[i][j];
    } // for i
  } // for j
  
  // Draw bar graphs
  for(int i=0; i<2; i++) {
    BarWidth=(dB(unsigned(WaveMax[i]-WaveMin[i]))-10)*3;
    if(BarWidth<0) BarWidth=0;
    if(BarWidth>MaxBarWidth) BarWidth=MaxBarWidth;
    
    if(BarWidth>=PeakVal[i]) {
      PeakVal[i]=BarWidth;
      PeakCnt[i]=0;
    } // if

    if(BarWidth>0          ) MGLCD.FillRect(BmpWidth+2,i*16+1,BmpWidth+2+BarWidth-1,i*16+14,1);
    if(BarWidth<MaxBarWidth) MGLCD.FillRect(BmpWidth+2+BarWidth,i*16+1,121,i*16+14,0);
    
    if(PeakVal[i]>2          ) MGLCD.FillRect(BmpWidth+2+PeakVal[i]-3,i*16+1,BmpWidth+2+PeakVal[i]-1,i*16+14,1);
    if(PeakVal[i]<MaxBarWidth) MGLCD.FillRect(BmpWidth+2+PeakVal[i],i*16+1,121,i*16+14,0);

    if(PeakVal[i]>0) {
      PeakCnt[i]++;
      if(PeakCnt[i]>=8) {
        PeakVal[i]--;
        PeakCnt[i]=0;
      } // if
    } // if
  } // for i
}
