/*
  S12232ZA_LEVEL_METER4.ino

  This sketch is for a level meter curcuit controlled by Arduino Uno.
  
  This sketch was developed by Synapse(Hiroshi Tanigawa) in 2013. This sketch is originally
  distributed at Team Schaft's Homepage. <http://www3.big.or.jp/~schaft/hardware>
  
  This sketch is free software: you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation, either version 2.1 of the License, or
  (at your option) any later version.
  
  This sketch is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this sketch.  If not, see <http://www.gnu.org/licenses/>.

*/

#include <MGLCD.h>
#include "kyoto.h"
#include "miku0.h"
#include "miku1.h"
#include "miku2.h"
#include "miku3.h"
#include "miku4.h"
#include "miku5.h"
#include "miku6.h"
#include "miku7.h"
#include "miku8.h"
#include "logo0.h"
#include "logo1.h"
#include "logo2.h"
#include "logo3.h"
#include "logo4.h"
#include "logo5.h"
#include "logo6.h"
#include "logo7.h"

#define CHARGE_PUMP_OSC_PIN 3

#define DIVISOR 4
#define WAVE_WIDTH 100
#define LOGO_WIDTH 20
#define LOGO_HEIGHT 112

const unsigned char *  pLogo[8];

const TLcdPinAssignTable PinAssignTable={
    A0_DI  : A3, // A0
    CS1_E1 : A0, // E1
    CS2_E2 : A1, // E2
    E      : MGLCD_UNUSED_PIN,
    RW     : A2,
    DB0    : 8 ,
    DB1    : 9 ,
    DB2    : 10,
    DB3    : 11,
    DB4    : 4 ,
    DB5    : 5 ,
    DB6    : 6,
    DB7    : 7
}; // PinAssignTable;

MGLCD_S12232ZA MGLCD(PinAssignTable);

void PrintSlowly(const char *p)
{
  while(*p!='\0') {
    MGLCD.print(*(p++));
    delay(100);
  } // while
} // PrintSlowly

void opening()
{
  MGLCD.ClearScreen();
  for(int i=90; i>=0; i--) {
    MGLCD.DrawBitmap(BMP_kyoto,i,1,MGLCD_ROM);
    MGLCD.Line(i,0,i,31,0);
    delay(1);
  } // int i
  
  MGLCD.SetInvertMode(MGLCD_INVERT);
  MGLCD.Locate(7,0); PrintSlowly(" PHONE METER ");
  MGLCD.SetInvertMode(MGLCD_NON_INVERT);
  MGLCD.Locate(7,1); PrintSlowly("This software");
  MGLCD.Locate(7,2); PrintSlowly("was designed");
  MGLCD.Locate(7,3); PrintSlowly("in Kyoto.");
  
  delay(3000);
  MGLCD.ClearScreen(); 
} // opening

void setup()
{
  // Enable Oscillator for charge pump
  pinMode(CHARGE_PUMP_OSC_PIN,OUTPUT);
  analogWrite(CHARGE_PUMP_OSC_PIN,127);
  
  // Enable pull-up registers of unused ports
  pinMode( 2,INPUT_PULLUP);
  pinMode(12,INPUT_PULLUP);
  
  // Turn on LED on port 13
  pinMode(13,OUTPUT);
  digitalWrite(13,HIGH);
  
  // Initialize LCD
  MGLCD.Reset();
  opening();
  
  // Initialize pLogo
  pLogo[0]=BMP_logo0;
  pLogo[1]=BMP_logo1;
  pLogo[2]=BMP_logo2;
  pLogo[3]=BMP_logo3;
  pLogo[4]=BMP_logo4;
  pLogo[5]=BMP_logo5;
  pLogo[6]=BMP_logo6;
  pLogo[7]=BMP_logo7;
} // setup

void loop()
{
  int wave[WAVE_WIDTH];
  int WaveMin;
  int WaveMax;

  // Get waves
  for(int i=0; i<WAVE_WIDTH; i++) {    
    wave[i]=analogRead(A4)+analogRead(A5);
  } // for i

  // Calculate WaveMin / WaveMax
  WaveMin=10000;
  WaveMax=0;  
  for(int i=0; i<WAVE_WIDTH; i++) {    
    if(wave[i]<WaveMin) WaveMin=wave[i];
    if(wave[i]>WaveMax) WaveMax=wave[i];
  } // for i
  
  // Display bar graphs
  {
    int val;
    
    val=(WaveMax-WaveMin)/DIVISOR;
    if(val<19) {
      MGLCD.DrawBitmap(BMP_miku0,0,0,MGLCD_ROM);      
    } else if(val<24) {
      MGLCD.DrawBitmap(BMP_miku1,0,0,MGLCD_ROM);      
    } else if(val<30) {
      MGLCD.DrawBitmap(BMP_miku2,0,0,MGLCD_ROM);      
    } else if(val<28) {
      MGLCD.DrawBitmap(BMP_miku3,0,0,MGLCD_ROM);      
    } else if(val<48) {
      MGLCD.DrawBitmap(BMP_miku4,0,0,MGLCD_ROM);      
    } else if(val<60) {
      MGLCD.DrawBitmap(BMP_miku5,0,0,MGLCD_ROM);      
    } else if(val<76) {
      MGLCD.DrawBitmap(BMP_miku6,0,0,MGLCD_ROM);      
    } else if(val<96) {
      MGLCD.DrawBitmap(BMP_miku7,0,0,MGLCD_ROM);
    } else {
      MGLCD.DrawBitmap(BMP_miku8,0,0,MGLCD_ROM);            
    } // if  
  }

  // Display Logo
  {
    static int LogoLine = 0;
    static int cnt = 0;
    unsigned char bmp[LOGO_WIDTH*4+2];
    
    bmp[0]=LOGO_WIDTH;
    bmp[1]=32;
    
    for(int y=0; y<4; y++) {
      for(int x=0; x<LOGO_WIDTH; x++) {
        bmp[y*LOGO_WIDTH+x+2]=pgm_read_byte(pLogo[LogoLine%8]+(LogoLine/8+y)%(LOGO_HEIGHT/8)*LOGO_WIDTH+x+2);
      } // for x
    } // for y
    MGLCD.DrawBitmap(bmp,102,0,MGLCD_RAM);
    
    if(++cnt>=4) {
      cnt=0;
      if(++LogoLine>=LOGO_HEIGHT) {
        LogoLine=0;
      } // if
    } // if
  }  
} // loop

